import rclpy
from rclpy.node import Node

from std_srvs.srv import Trigger

from std_msgs.msg import Float32
from sensor_msgs.msg import Joy
from geometry_msgs.msg import TwistStamped
from moveit_msgs.msg import PlanningScene

from enum import Enum

class Buttons(Enum):
    CROSS = 0
    CIRCLE = 1
    TRIANGLE = 2
    SQUARE = 3
    L1 = 4
    R1 = 5
    L2 = 6
    R2 = 7
    SELECT = 8
    START = 9
    CENTER = 10
    L3 = 11
    R3 = 12

class Axes(Enum):
    LEFT_X = 0
    LEFT_Y = 1
    L1 = 2
    RIGHT_X = 3
    RIGHT_Y = 4
    R1 = 5
    DPAD_UPDOWN = 6
    DPAD_LEFTRIGHT = 7

class JoyMove(Node):
    def __init__(self):
        super().__init__('joystickmove')
        self._logger = self.get_logger()
        self._logger.info('joystick init')

        JOY_TOPIC = '/joy'
        TWIST_TOPIC = '/servo_server/delta_twist_cmds'
        # JOINT_TOPIC = '/servo_server/delta_joint_cmds'
        # EE = 'joint_6'
        # BASE = 'base_link'

        self.create_subscription(Joy, '/joy', self._joy_cb, rclpy.qos.qos_profile_sensor_data)

        self.base_pub = self.create_publisher(Float32, JOY_TOPIC, 10)
        self.twist_pub = self.create_publisher(TwistStamped, TWIST_TOPIC, 10);
        # self.collision_pub = self.create_publisher(PlanningScene, "/planning_scene", 10);

        self.servo_client = self.create_client(Trigger, '/servo_server/start_servo')
        # Check if the a service is available
        while not self.servo_client.wait_for_service(timeout_sec=1.0):
            self.get_logger().info('servo_server not available, waiting again...')
        self._logger.info('servo_server connected.')

        req = Trigger.Request()
        self.servo_client.call_async(req)


    def _joy_cb(self, msg):
        axes = msg.axes
        buttons = msg.buttons

        ts = TwistStamped()

        ts.twist.linear.z = axes[Axes.RIGHT_Y.value]
        ts.twist.linear.y = axes[Axes.RIGHT_X.value]

        lin_x_right = -0.5 * axes[Axes.R1.value] # - AXIS_DEFAULTS.at(R1));
        lin_x_left = 0.5 * axes[Axes.L1.value] # - AXIS_DEFAULTS.at(L1));
        ts.twist.linear.x = lin_x_right + lin_x_left

        ts.twist.angular.y = axes[Axes.LEFT_Y.value]
        ts.twist.angular.x = axes[Axes.LEFT_X.value]

        roll_positive = buttons[Buttons.R2.value]
        roll_negative = -1 * buttons[Buttons.L2.value]
        ts.twist.angular.z = float(roll_positive + roll_negative)

        ts.header.stamp = self.get_clock().now().to_msg()

        self.twist_pub.publish(ts)


def main(args=None):
    rclpy.init(args=args)

    joy_move = JoyMove()
    rclpy.spin(joy_move)

    joy_move.destroy_node()
    rclpy.shutdown()


if __name__ == '__main__':
    main()
